@@ -3,7 +3,7 @@
  *
  * pdfresurrect - PDF history extraction tool
  *
- * Copyright (C) 2008 Matt Davis (enferex) of 757Labs (www.757labs.com)
+ * Copyright (C) 2008, 2009 Matt Davis (enferex) of 757Labs (www.757labs.com)
  *
  * pdf.c is part of pdfresurrect.
  * pdfresurrect is free software: you can redistribute it and/or modify
@@ -35,6 +35,15 @@ static int is_valid_xref(FILE *fp, pdf_t *pdf, xref_t *xref);
 static void load_xref_entries(FILE *fp, xref_t *xref);
 static void load_xref_from_plaintext(FILE *fp, xref_t *xref);
 static void load_xref_from_stream(FILE *fp, xref_t *xref);
+static void get_xref_linear_skipped(FILE *fp, xref_t *xref);
+static void resolve_linearized_pdf(pdf_t *pdf);
+
+
+static pdf_creator_t *new_creator(int *n_elements);
+static void load_creator(FILE *fp, pdf_t *);
+static void load_creator_from_buf(xref_t *xref, const char *buf);
+static void load_creator_from_xml(xref_t *xref, const char *buf);
+static void load_creator_from_old_format(xref_t *xref, const char *buf);
 
 static char *get_object_from_here(FILE *fp, size_t *size, int *is_stream);
 
@@ -49,7 +58,10 @@ static void add_kid(int id, xref_t *xref);
 static void load_kids(FILE *fp, int pages_id, xref_t *xref);
 
 static const char *get_type(FILE *fp, int obj_id, const xref_t *xref);
-static int get_page(int obj_id, const xref_t *xref);
+/* static int get_page(int obj_id, const xref_t *xref); */
+static char *get_header(FILE *fp);
+
+static char *decode_text_string(const char *str, size_t str_len);
 
 
 /*
@@ -90,6 +102,7 @@ void pdf_delete(pdf_t *pdf)
 
     for (i=0; i<pdf->n_xrefs; i++)
     {
+        free(pdf->xrefs[i].creator);
         free(pdf->xrefs[i].entries);
         free(pdf->xrefs[i].kids);
     }
@@ -102,19 +115,43 @@ void pdf_delete(pdf_t *pdf)
 
 int pdf_is_pdf(FILE *fp)
 {
-    char magic[5];
+    int   is_pdf;
+    char *header;
 
-    fseek(fp, 0, SEEK_SET);
-    fread(magic, 5, 1, fp);
+    header = get_header(fp);
 
-    return (memcmp(magic, "%PDF-", 5) == 0);
+    if (header && strstr(header, "%PDF-"))
+      is_pdf = 1;
+    else 
+      is_pdf = 0;
+
+    free(header);
+    return is_pdf;
+}
+
+
+void pdf_get_version(FILE *fp, pdf_t *pdf)
+{
+    char *header, *c;
+
+    header = get_header(fp);
+
+    /* Locate version string start and make sure we dont go past header */
+    if ((c = strstr(header, "%PDF-")) && 
+        (c + strlen("%PDF-M.m") + 2))
+    {
+        pdf->pdf_major_version = atoi(c + strlen("%PDF-"));
+        pdf->pdf_minor_version = atoi(c + strlen("%PDF-M."));
+    }
+
+    free(header);
 }
 
 
 int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
 {
+    int  i, ver, is_linear;
     long pos, pos_count;
-    int  i, ver;
     char x, *c, buf[256];
     
     c = NULL;
@@ -127,15 +164,18 @@ int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
         ++pdf->n_xrefs;
 
     /* Load in the start/end positions */
-    ver = 1;
     fseek(fp, 0, SEEK_SET);
     pdf->xrefs = calloc(1, sizeof(xref_t) * pdf->n_xrefs);
+    ver = 1;
     for (i=0; i<pdf->n_xrefs; i++)
     {
         while (fgets(buf, 256, fp))
           if ((c = strstr(buf, "%%EOF")))
             break;
-        
+
+        /* Set and increment the version */
+        pdf->xrefs[i].version = ver++;
+
         /* Seek to %%EOF */
         fseek(fp, c - buf - strlen(buf), SEEK_CUR);
         pos = ftell(fp);
@@ -155,30 +195,51 @@ int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
         /* xref start position */
         pdf->xrefs[i].start = atol(c);
 
-        /* xref end position */
-        pos = ftell(fp);
-        fseek(fp, pdf->xrefs[i].start, SEEK_SET);
-        while (fgets(buf, 256, fp))
-          if ((c = strstr(buf, "%%EOF")))
-            break;
+        /* If xref is 0 handle linear xref table */
+        if (pdf->xrefs[i].start == 0)
+          get_xref_linear_skipped(fp, &pdf->xrefs[i]);
 
-        fseek(fp, c - buf - strlen(buf), SEEK_CUR);
-        pdf->xrefs[i].end = ftell(fp);
+        /* Non-linear, normal operation, so just find the end of the xref */
+        else
+        {
+            /* xref end position */
+            pos = ftell(fp);
+            fseek(fp, pdf->xrefs[i].start, SEEK_SET);
+            while (fgets(buf, 256, fp))
+              if ((c = strstr(buf, "%%EOF")))
+                break;
 
-        /* Look for next EOF and xref data */
-        fseek(fp, pos, SEEK_SET);
+            fseek(fp, c - buf - strlen(buf), SEEK_CUR);
+            pdf->xrefs[i].end = ftell(fp);
+
+            /* Look for next EOF and xref data */
+            fseek(fp, pos, SEEK_SET);
+        }
+
+        /* Check validity */
         if (!is_valid_xref(fp, pdf, &pdf->xrefs[i]))
         {
+            is_linear = pdf->xrefs[i].is_linear;
             memset(&pdf->xrefs[i], 0, sizeof(xref_t));
+            pdf->xrefs[i].is_linear = is_linear;
             continue;
         }
-        else
-          pdf->xrefs[i].version = ver++;
 
         /*  Load the entries from the xref */
         load_xref_entries(fp, &pdf->xrefs[i]);
     }
 
+    /* Now we have all xref tables, if this is linearized, we need
+     * to make adjustments so that things spit out properly
+     */
+    if (pdf->xrefs[0].is_linear)
+      resolve_linearized_pdf(pdf);
+
+    /* Ok now we have all xref data.  Go through those versions of the 
+     * PDF and try to obtain creator information
+     */
+    load_creator(fp, pdf);
+
     return pdf->n_xrefs;
 }
 
@@ -195,7 +256,7 @@ void pdf_load_pages_kids(FILE *fp, pdf_t *pdf)
     /* Load all kids for all xref tables (versions) */
     for (i=0; i<pdf->n_xrefs; i++)
     {
-        if (pdf->xrefs[i].version)
+        if (pdf->xrefs[i].version && (pdf->xrefs[i].end != 0))
         {
             fseek(fp, pdf->xrefs[i].start, SEEK_SET);
             while (fgetc(fp) != 't')
@@ -238,28 +299,40 @@ char pdf_get_object_status(
     int          xref_idx,
     int          entry_idx)
 {
-    int           i, j;
-    xref_entry_t *prev, *curr;
+    int                 i, curr_ver;
+    const xref_t       *prev_xref;
+    const xref_entry_t *prev, *curr;
 
     curr = &pdf->xrefs[xref_idx].entries[entry_idx];
+    curr_ver = pdf->xrefs[xref_idx].version;
+
+    if (curr_ver == 1)
+      return 'A';
 
     /* Deleted (freed) */
     if (curr->f_or_n == 'f')
       return 'D';
 
-    /* Get previous entry */
+    /* Get previous version */
+    prev_xref = NULL;
+    for (i=xref_idx; i>-1; --i)
+      if (pdf->xrefs[i].version < curr_ver)
+      {
+          prev_xref = &pdf->xrefs[i];
+          break;
+      }
+
+    if (!prev_xref)
+      return '?';
+
+    /* Locate the object in the previous one that matches current one */
     prev = NULL;
-    for (i=xref_idx-1; i>-1; i--)
-    {
-        for (j=0; j<pdf->xrefs[i].n_entries; j++)
-        {
-            if (pdf->xrefs[i].entries[j].obj_id == curr->obj_id)
-            {
-                prev = &pdf->xrefs[i].entries[j];
-                break;
-            }
-        }
-    }
+    for (i=0; i<prev_xref->n_entries; ++i)
+      if (prev_xref->entries[i].obj_id == curr->obj_id)
+      {
+          prev = &prev_xref->entries[i];
+          break;
+      }
 
     /* Added in place of a previously freed id */
     if (!prev || ((prev->f_or_n == 'f') && (curr->f_or_n == 'n')))
@@ -311,12 +384,12 @@ void pdf_summarize(
     const char  *name,
     pdf_flag_t   flags)
 {
-    int   i, j, page, n_versions;
+    int   i, j, page, n_versions, n_entries;
     FILE *dst, *out;
     char *dst_name, *c;
 
-    dst_name = NULL;
     dst = NULL;
+    dst_name = NULL;
 
     if (name)
     {
@@ -337,42 +410,47 @@ void pdf_summarize(
     /* Send output to file or stdout */
     out = (dst) ? dst : stdout;
 
-    /* Count verions */
-    n_versions = 0;
-    for (i=0; i<pdf->n_xrefs; i++)
-      if (pdf->xrefs[i].version)
-        ++n_versions;
+    /* Count versions */
+    n_versions = pdf->n_xrefs;
+    if (pdf->xrefs[0].is_linear)
+      --n_versions;
+
+    /* Ignore bad xref entry */
+    for (i=1; i<pdf->n_xrefs; ++i)
+      if (pdf->xrefs[i].end == 0)
+        --n_versions;
+
+    /* If we have no valid versions but linear, count that */
+    if (!n_versions && pdf->xrefs[0].is_linear)
+      n_versions = 1;
 
     /* Compare each object (if we dont have xref streams) */
+    n_entries = 0;
     for (i=0; !(const int)pdf->has_xref_streams && i<pdf->n_xrefs; i++)
     {
-        if (pdf->xrefs[i].version)
+        if (flags & PDF_FLAG_QUIET)
+          continue;
+
+        for (j=0; j<pdf->xrefs[i].n_entries; j++)
         {
-            if (flags & PDF_FLAG_QUIET)
-              continue;
+            ++n_entries;
+            fprintf(out,
+                    "%s: --%c-- Version %d -- Object %d (%s)",
+                    pdf->name,
+                    pdf_get_object_status(pdf, i, j),
+                    pdf->xrefs[i].version,
+                    pdf->xrefs[i].entries[j].obj_id,
+                    get_type(fp, pdf->xrefs[i].entries[j].obj_id,
+                             &pdf->xrefs[i]));
 
-            for (j=0; j<pdf->xrefs[i].n_entries; j++)
-            {
-                if (!pdf->xrefs[i].entries[j].obj_id)
-                  continue;
+            /* TODO
+            page = get_page(pdf->xrefs[i].entries[j].obj_id, &pdf->xrefs[i]);
+            */
 
-                fprintf(out,
-                        "%s: --%c-- Version %d -- Object %d (%s)",
-                        pdf->name,
-                        pdf_get_object_status(pdf, i, j),
-                        pdf->xrefs[i].version,
-                        pdf->xrefs[i].entries[j].obj_id,
-                        get_type(fp, pdf->xrefs[i].entries[j].obj_id,
-                                 &pdf->xrefs[i]));
-
-                page = get_page(pdf->xrefs[i].entries[j].obj_id,
-                                &pdf->xrefs[i]);
-
-                if (page)
-                  fprintf(out, " Page(%d)\n", page);
-                else
-                  fprintf(out, "\n");
-            }
+            if (0 /*page*/)
+              fprintf(out, " Page(%d)\n", page);
+            else
+              fprintf(out, "\n");
         }
     }
 
@@ -383,9 +461,9 @@ void pdf_summarize(
          * If we have a 1.5 PDF using streams for xref, we have not objects
          * to display, so let the user know whats up.
          */
-        if (pdf->has_xref_streams)
+        if (pdf->has_xref_streams || !n_entries)
            fprintf(out,
-               "%s: This PDF contains cross reference streams.\n"
+               "%s: This PDF contains potential cross reference streams.\n"
                "%s: An object summary is not available.\n",
                pdf->name,
                pdf->name);
@@ -396,13 +474,27 @@ void pdf_summarize(
                 pdf->name,
                 n_versions);
 
+        /* Count entries for summary */
         if (!pdf->has_xref_streams)
           for (i=0; i<pdf->n_xrefs; i++)
-            if (pdf->xrefs[i].version)
-              fprintf(out,
-                      "Version %d -- %d objects\n",
-                      pdf->xrefs[i].version, 
-                      pdf->xrefs[i].n_entries);
+          {
+              if (pdf->xrefs[i].is_linear)
+                continue;
+
+              n_entries = pdf->xrefs[i].n_entries;
+
+              /* If we are a linearized PDF, all versions are made from those
+               * objects too.  So count em'
+               */
+              if (pdf->xrefs[0].is_linear)
+                n_entries += pdf->xrefs[0].n_entries; 
+
+              if (pdf->xrefs[i].version && n_entries)
+                fprintf(out,
+                        "Version %d -- %d objects\n",
+                        pdf->xrefs[i].version, 
+                        n_entries);
+           }
     }
     else /* Quiet output */
       fprintf(out, "%s: %d\n", pdf->name, n_versions);
@@ -415,6 +507,23 @@ void pdf_summarize(
 }
 
 
+/* Returns '1' if we successfully display data (means its probably not xml) */
+int pdf_display_creator(const pdf_t *pdf, int xref_idx)
+{
+    int i;
+
+    if (!pdf->xrefs[xref_idx].creator)
+      return 0;
+
+    for (i=0; i<pdf->xrefs[xref_idx].n_creator_entries; ++i)
+      printf("%s: %s\n",
+             pdf->xrefs[xref_idx].creator[i].key,
+             pdf->xrefs[xref_idx].creator[i].value);
+
+    return (i > 0);
+}
+
+
 /* Checks if the xref is valid and sets 'is_stream' flag if the xref is a
  * stream (PDF 1.5 or higher) */
 static int is_valid_xref(FILE *fp, pdf_t *pdf, xref_t *xref)
@@ -422,12 +531,13 @@ static int is_valid_xref(FILE *fp, pdf_t *pdf, xref_t *xref)
     int   is_valid;
     long  start;
     char *c, buf[16];
-
+    
+    memset(buf, 0, sizeof(buf));
     is_valid = 0;
     start = ftell(fp);
     fseek(fp, xref->start, SEEK_SET);
-    fgets(buf, 16, fp);
 
+    fgets(buf, 16, fp);
     if (strncmp(buf, "xref", strlen("xref")) == 0)
       is_valid = 1;
 
@@ -545,6 +655,282 @@ static void load_xref_from_stream(FILE *fp, xref_t *xref)
 }
 
 
+static void get_xref_linear_skipped(FILE *fp, xref_t *xref)
+{
+    int   err;
+    char *c, ch, buf[256];
+
+    if (xref->start != 0)
+      return;
+
+    /* Special case (Linearized PDF with initial startxref at 0) */
+    xref->is_linear = 1;
+
+    /* Find the next xref table that was skipped over and 
+     * acquire that.
+     */
+    c = NULL;
+    while (fgets(buf, 256, fp))
+      if ((c = strstr(buf, "%%EOF")))
+        break;
+
+    if (!c)
+      return;
+
+    /* Seek to %%EOF */
+    fseek(fp, c - buf - strlen(buf), SEEK_CUR);
+    xref->end = ftell(fp);
+
+    /* Locate the trailer */ 
+    err = 0; 
+    while (!(err = ferror(fp)) && fread(buf, 1, 8, fp))
+    {
+        if (strncmp(buf, "trailer", strlen("trailer")) == 0)
+          break;
+        else if ((ftell(fp) - 9) < 0)
+          return;
+
+        fseek(fp, -9, SEEK_CUR);
+    }
+
+    if (err)
+      return;
+
+    /* If we found 'trailer' look backwards for 'xref' */
+    ch = 0;
+    while (!ferror(fp) && ((ch = fgetc(fp)) != 'x'))
+      fseek(fp, -2, SEEK_CUR);
+
+    if (ch == 'x')
+    {
+        xref->start = ftell(fp) - 1;
+        fseek(fp, -1, SEEK_CUR);
+    }
+
+    /* Now continue to next eof ... */
+    fseek(fp, xref->start, SEEK_SET);
+}
+
+
+/* This must only be called after all xref and entries have been acquired */
+static void resolve_linearized_pdf(pdf_t *pdf)
+{
+    int    i;
+    xref_t buf;
+
+    if (!pdf->xrefs[0].is_linear)
+      return;
+
+    /* Swap Linear with Version 1 */
+    buf = pdf->xrefs[0];
+    pdf->xrefs[0] = pdf->xrefs[1];
+    pdf->xrefs[1] = buf;
+
+    /* Resolve is_linear flag and version */
+    pdf->xrefs[0].is_linear = 1;
+    pdf->xrefs[0].version = 1;
+    pdf->xrefs[1].is_linear = 0;
+    pdf->xrefs[1].version = 1;
+
+    /* Adjust the other version values now */
+    for (i=2; i<pdf->n_xrefs; ++i)
+      --pdf->xrefs[i].version;
+}
+
+
+static pdf_creator_t *new_creator(int *n_elements)
+{
+    pdf_creator_t *daddy;
+
+    static const pdf_creator_t creator_template[] = 
+    {
+        {"Title",        ""},
+        {"Author",       ""},
+        {"Subject",      ""},
+        {"Keywords",     ""},
+        {"Creator",      ""},
+        {"Producer",     ""},
+        {"CreationDate", ""},
+        {"ModDate",      ""},
+        {"Trapped",      ""},
+    };
+
+    daddy = malloc(sizeof(creator_template));
+    memcpy(daddy, creator_template, sizeof(creator_template));
+
+    if (n_elements)
+      *n_elements = sizeof(creator_template) / sizeof(creator_template[0]);
+
+    return daddy;
+}
+
+
+#define END_OF_TRAILER(_c, _st, _fp) \
+{                                    \
+    if (_c == '>')                   \
+    {                                \
+        fseek(_fp, _st, SEEK_SET);   \
+        continue;                    \
+    }                                \
+}
+static void load_creator(FILE *fp, pdf_t *pdf)
+{
+    int    i, buf_idx;
+    char   c, *buf, obj_id_buf[32];
+    long   start;
+    size_t sz;
+
+    start = ftell(fp);
+
+    /* For each PDF version */
+    for (i=0; i<pdf->n_xrefs; ++i)
+    {
+        if (!pdf->xrefs[i].version)
+          continue;
+
+        /* Find trailer */
+        fseek(fp, pdf->xrefs[i].start, SEEK_SET);
+        while (fgetc(fp) != 't')
+            ; /* Iterate to "trailer" */
+
+        /* Look for "<< ....... /Info ......" */
+        while ((c = fgetc(fp)) != '>')
+          if ((c == '/') && (fgetc(fp) == 'I') && ((fgetc(fp) == 'n')))
+            break;
+
+        /* Could not find /Info in trailer */
+        END_OF_TRAILER(c, start, fp);
+
+        while (!isspace(c = fgetc(fp)) && (c != '>'))
+            ; /* Iterate to first white space /Info<space><data> */
+
+        /* No space between /Info and it's data */
+        END_OF_TRAILER(c, start, fp);
+
+        while (isspace(c = fgetc(fp)) && (c != '>'))
+            ; /* Iterate right on top of first non-whitespace /Info data */
+
+        /* No data for /Info */
+        END_OF_TRAILER(c, start, fp);
+
+        /* Get obj id as number */
+        buf_idx = 0;
+        obj_id_buf[buf_idx++] = c;
+        while (!isspace(c = fgetc(fp)) && (c != '>'))
+          obj_id_buf[buf_idx++] = c;
+
+        END_OF_TRAILER(c, start, fp);
+
+        obj_id_buf[buf_idx] = '\0';
+     
+        /* Get the object for the creator data.  If linear, try both xrefs */ 
+        buf = get_object(fp, atoll(obj_id_buf), &pdf->xrefs[i], &sz, NULL);
+        if (!buf && pdf->xrefs[i].is_linear && (i+1 < pdf->n_xrefs))
+          buf = get_object(fp, atoll(obj_id_buf), &pdf->xrefs[i+1], &sz, NULL);
+
+        load_creator_from_buf(&pdf->xrefs[i], buf);
+        free(buf);
+    }
+
+    fseek(fp, start, SEEK_SET);
+}
+
+
+static void load_creator_from_buf(xref_t *xref, const char *buf)
+{
+    int   is_xml;
+    char *c;
+
+    if (!buf)
+      return;
+
+    /* Check to see if this is xml or old-school */
+    if ((c = strstr(buf, "/Type")))
+      while (*c && !isspace(*c))
+        ++c;
+
+    /* Probably "Metadata" */
+    is_xml = 0;
+    if (c && (*c == 'M'))
+      is_xml = 1;
+
+    /* Is the buffer XML(PDF 1.4+) or old format? */
+    if (is_xml)
+      load_creator_from_xml(xref, buf);
+    else
+      load_creator_from_old_format(xref, buf);
+}
+
+
+static void load_creator_from_xml(xref_t *xref, const char *buf)
+{
+    /* TODO */
+}
+
+
+static void load_creator_from_old_format(xref_t *xref, const char *buf)
+{
+    int            i, n_eles, length, is_escaped;
+    char          *c, *ascii, *start;
+    pdf_creator_t *info;
+
+    info = new_creator(&n_eles);
+
+    for (i=0; i<n_eles; ++i)
+    {
+        if (!(c = strstr(buf, info[i].key)))
+          continue;
+
+        /* Find the value (skipping whitespace) */
+        c += strlen(info[i].key);
+        while (isspace(*c))
+          ++c;
+
+        /* If looking at the start of a pdf token, we have gone too far */
+        if (*c == '/')
+          continue;
+
+        /* Find the end of the value */
+        start = c;
+        length = is_escaped = 0;
+        while (c && ((*c != '\r') && (*c != '\n') && (*c != '<')))
+        {
+            /* Bail out if we see an un-escaped ')' closing character */
+            if (!is_escaped && (*c == ')'))
+              break;
+            else if (*c == '\\')
+              is_escaped = 1;
+            else
+              is_escaped = 0;
+
+            ++c;
+            ++length;
+        }
+
+        if (length == 0)
+          continue;
+
+        /* Add 1 to length so it gets the closing ')' when we copy */
+        if (length)
+          length += 1;
+        length = (length > KV_MAX_VALUE_LENGTH) ? KV_MAX_VALUE_LENGTH : length;
+        strncpy(info[i].value, start, length);
+        info[i].value[KV_MAX_VALUE_LENGTH - 1] = '\0';
+    }
+
+    /* Go through the values and convert if encoded */
+    for (i=0; i<n_eles; ++i)
+      if ((ascii = decode_text_string(info[i].value, strlen(info[i].value))))
+      {
+          strncpy(info[i].value, ascii, strlen(info[i].value));
+          free(ascii);
+      }
+
+    xref->creator = info;
+    xref->n_creator_entries = n_eles;
+}
+
+
 /* Returns object data at the start of the file pointer
  * This interfaces to 'get_object'
  */
@@ -774,11 +1160,9 @@ static const char *get_type(FILE *fp, int obj_id, const xref_t *xref)
 }
 
 
+/* TODO
 static int get_page(int obj_id, const xref_t *xref)
 {
-    /* TODO */
-    return 0;
-    /*
     int i;
 
     for (i=0; i<xref->n_kids; i++)
@@ -786,5 +1170,68 @@ static int get_page(int obj_id, const xref_t *xref)
         break;
 
     return i;
-    */
+}
+*/
+
+
+static char *get_header(FILE *fp)
+{
+    long start;
+
+    /* First 1024 bytes of doc must be header (1.7 spec pg 1102) */
+    char *header;
+
+    header = calloc(1, 1024);
+    
+    start = ftell(fp);
+    fseek(fp, 0, SEEK_SET);
+    fread(header, 1024, 1, fp);
+    fseek(fp, start, SEEK_SET);
+    
+    return header;
+}
+
+
+static char *decode_text_string(const char *str, size_t str_len)
+{
+    int   idx, is_hex, is_utf16be, ascii_idx;
+    char *ascii, hex_buf[5] = {0};
+
+    is_hex = is_utf16be = idx = ascii_idx = 0;
+
+    /* Regular encoding */
+    if (str[0] == '(')
+    {
+        ascii = strdup(str);
+        return ascii;
+    }
+    else if (str[0] == '<')
+    {
+        is_hex = 1;
+        ++idx;
+    }
+    
+    /* Text strings can be either PDFDocEncoding or UTF-16BE */
+    if (is_hex && (str_len > 5) && 
+        (str[idx] == 'F') && (str[idx+1] == 'E') &&
+        (str[idx+2] == 'F') && (str[idx+3] == 'F'))
+    {
+        is_utf16be = 1;
+        idx += 4;
+    }
+    else
+      return NULL;
+
+    /* Now decode as hex */
+    ascii = malloc(str_len);
+    for ( ; idx<str_len; ++idx)
+    {
+        hex_buf[0] = str[idx++];
+        hex_buf[1] = str[idx++];
+        hex_buf[2] = str[idx++];
+        hex_buf[3] = str[idx];
+        ascii[ascii_idx++] = strtol(hex_buf, NULL, 16);
+    }
+
+    return ascii;
 }
